@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef _ASNER_H
-#define _ASNER_H
+#ifndef PTLIB_ASNER_H
+#define PTLIB_ASNER_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -51,9 +51,6 @@ class PXMLElement;
 #undef      P_INCLUDE_XER
 #endif
 
-#ifndef _PTLIB_H
-#include <ptlib.h>
-#endif
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -996,4 +993,7 @@ class PASN_Stream : public PBYTEArray
 #include "asnxer.h"
 #endif
 
-#endif // _ASNER_H
+#endif // PTLIB_ASNER_H
+
+
+// End Of File ///////////////////////////////////////////////////////////////
