@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef _IPACL_H
-#define _IPACL_H
+#ifndef PTLIB_IPACL_H
+#define PTLIB_IPACL_H
 
 
 #include <ptlib/sockets.h>
@@ -348,7 +348,7 @@ class PIpAccessControlList : public PIpAccessControlList_base
 };
 
 
-#endif  // _IPACL_H
+#endif  // PTLIB_IPACL_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
