@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 22848 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-06-12 06:26:03 +0000 (Fri, 12 Jun 2009) $
  */
 
 #ifndef PTLIB_VXML_H
@@ -274,8 +274,6 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
 
     PXMLElement * FindForm(const PString & id);
 
-    //friend class PVXMLChannel;
-
     PSyncPoint waitForEvent;
 
     PMutex sessionMutex;
@@ -308,7 +306,7 @@ class PVXMLSession : public PIndirectChannel, public PVXMLChannelInterface
     PBoolean forceEnd;
 
     PString mediaFormat;
-    PVXMLChannel * vxmlChannel;
+    PVXMLChannel * & vxmlChannel;
 
     PTextToSpeech * textToSpeech;
     PBoolean autoDeleteTextToSpeech;
