@@ -26,14 +26,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
 
-#ifndef _PCONFIG
-#define _PCONFIG
+#ifndef PTLIB_CONFIG_H
+#define PTLIB_CONFIG_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -519,6 +519,6 @@ class PConfig : public PObject
 
 #endif // P_CONFIG_FILE
 
-#endif
+#endif // PTLIB_CONFIG_H
 
 // End Of File ///////////////////////////////////////////////////////////////
