@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22069 $
+ * $Revision: 23340 $
  * $Author: rjongbloed $
- * $Date: 2009-02-16 01:01:51 +0000 (Mon, 16 Feb 2009) $
+ * $Date: 2009-09-05 03:25:03 +0000 (Sat, 05 Sep 2009) $
  */
 
 #ifndef PTLIB_CONTAIN_H
@@ -236,6 +236,8 @@ typedef long PInt32;
 
 #endif // !_WIN32_WCE 
 
+#define _hptr(x) (PINDEX)(x)
+
 
 class PWin32Overlapped : public OVERLAPPED
 {
@@ -288,29 +290,13 @@ class RegistryKey
     #pragma warning(disable:4127 4706)
   #endif
 
-  #if defined(P_WINSOCKv1)
-
-    #include <winsock.h>
-
-  #else // P_WINSOCKv1
-
-    ///IPv6 support
-    // Needed for for IPv6 socket API. Must be included before "windows.h"
-    #include <winsock2.h> // Version 2 of windows socket
-    #include <ws2tcpip.h> // winsock2 is not complete, ws2tcpip add some defines such as IP_TOS
-
-    #if P_HAS_IPV6 && !defined IPPROTO_IPV6
-      #include "tpipv6.h"  // For IPv6 Tech Preview.
-    #endif
-
-  #endif // P_WINSOCKv1
+  #include <winsock2.h> // Version 2 of windows socket
+  #include <ws2tcpip.h> // winsock2 is not complete, ws2tcpip add some defines such as IP_TOS
 
   #if defined(_MSC_VER)
     #pragma warning(pop)
   #endif
 
-  #define PIPX
-
   typedef int socklen_t;
 
 #endif  // P_KNOCKOUT_WINSOCK2
