@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
- * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Revision: 22552 $
+ * $Author: csoutheren $
+ * $Date: 2009-05-07 08:53:20 +0000 (Thu, 07 May 2009) $
  */
 
 #ifndef PTLIB_FACTORY_H
@@ -406,6 +406,33 @@ class PFactory : PFactoryBase
   namespace PWLibFactoryLoader { int AbstractType##_##KeyType##_loader; }
 
 
+/** This macro is used to create a factory.
+    This is mainly used for factories that exist inside a library and works in
+    conjunction with the PFACTORY_LOAD() macro.
+
+    When a factory is contained wholly within a single compilation module of
+    a library, it is typical that a linker does not include ANY of the code in
+    that module. To avoid this the header file that declares the abstract type
+    should include a PFACTORY_LOAD() macro call for all concrete classes that
+    are in the library. Then whan an application includes the abstract types
+    header, it will force the load of all the possible concrete classes.
+ */
+#define PFACTORY_CREATE(factory, ConcreteType, keyValue, singleton) \
+  namespace PFactoryLoader { \
+    int ConcreteType##_link() { return 0; } \
+    factory::Worker<ConcreteType> ConcreteType##_instance(keyValue, singleton); \
+  }
+
+/* This macro is used to force linking of factories.
+   See PFACTORY_CREATE() for more information
+ */
+#define PFACTORY_LOAD(ConcreteType) \
+  namespace PFactoryLoader { \
+    extern int ConcreteType##_link(); \
+    int const ConcreteType##_loader = ConcreteType##_link(); \
+  }
+
+
 #endif // PTLIB_FACTORY_H
 
 
