@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 22378 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-04-07 03:20:13 +0000 (Tue, 07 Apr 2009) $
  */
 
 #ifndef PTLIB_SERVICEPROCESS_H
@@ -81,7 +81,12 @@ class PSystemLog : public PObject, public iostream {
     /// Create a system log stream
     PSystemLog(
      Level level   ///< only messages at this level or higher will be logged
-    );
+    ) : iostream(cout.rdbuf())
+      , logLevel(level)
+    { 
+      buffer.log = this; 
+      init(&buffer); 
+    }
 
     /// Destroy the string stream, deleting the stream buffer
     ~PSystemLog() { flush(); }
@@ -115,8 +120,8 @@ class PSystemLog : public PObject, public iostream {
   //@}
 
   private:
-    PSystemLog(const PSystemLog & other);
-    PSystemLog & operator=(const PSystemLog &);
+    PSystemLog(const PSystemLog & other) : PObject(other), iostream(cout.rdbuf()) { }
+    PSystemLog & operator=(const PSystemLog &) { return *this; }
 
     class Buffer : public streambuf {
       public:
