@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20986 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-09-15 12:01:50 +0000 (Mon, 15 Sep 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-
-#define _PSYNCPOINTACK
+#ifndef PTLIB_SYNCTHRD_H
+#define PTLIB_SYNCTHRD_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -403,4 +403,7 @@ class PWriteWaitAndSignal {
 };
 
 
+#endif // PTLIB_SYNCTHRD_H
+
+
 // End Of File ///////////////////////////////////////////////////////////////
