@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21869 $
- * $Author: hfriederich $
- * $Date: 2008-12-25 12:58:26 +0000 (Thu, 25 Dec 2008) $
+ * $Revision: 23120 $
+ * $Author: rjongbloed $
+ * $Date: 2009-07-21 23:31:08 +0000 (Tue, 21 Jul 2009) $
  */
 
 #ifndef _PMACHDEP_H
@@ -342,6 +342,7 @@ typedef int socklen_t;
 #endif
  
 #define HAS_IFREQ
+#define _POSIX_MONOTONIC_CLOCK
 
 // hf, 2008-12-25: apparently, earlier versions of 10.5.x needed a version 
 // without parameters. however, something changed such that the 'old' version
