@@ -12,19 +12,18 @@
  * compliance with the License. You may obtain a copy of the License at
  * http://www.mozilla.org/MPL/
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef __SHMVIDEO_H__
-#define __SHMVIDEO_H__
+#ifndef PTLIB_SHMVIDEO_H
+#define PTLIB_SHMVIDEO_H
 
 #define P_FORCE_STATIC_PLUGIN
 
-#ifndef _PTLIB_H
-#include <ptlib.h>
-#endif
+
+#if P_SHM_VIDEO
 
 #include <ptclib/delaychan.h>
 
@@ -176,5 +175,6 @@ class PVideoInputDevice_Shm : public PVideoInputDevice
     void *shmPtr;
 };
 
+#endif
 
-#endif /* __SHMVIDEO_H__ */
+#endif // PTLIB_SHMVIDEO_H
