@@ -30,9 +30,9 @@
  * Contributor(s): Guilhem Tardy (gtardy@salyens.com)
  *  Nicola Orru' <nigu@itadinanta.it>
  *
- * $Revision: 22071 $
+ * $Revision: 23018 $
  * $Author: rjongbloed $
- * $Date: 2009-02-16 02:56:21 +0000 (Mon, 16 Feb 2009) $
+ * $Date: 2009-07-01 00:13:04 +0000 (Wed, 01 Jul 2009) $
  */
 
 #pragma implementation "vidinput_v4l2.h"
@@ -79,7 +79,7 @@ protected:
 };
 
 
-PMutex creationMutex;
+static PMutex creationMutex;
 static 
 V4L2Names & GetNames()
 {
