@@ -27,9 +27,9 @@
  * Original copyright notice appears below
  *
  * $Id: pssl.cxx 19752 2008-03-15 15:30:24Z hfriederich $
- * $Revision: 20385 $
+ * $Revision: 23262 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2009-08-25 00:25:25 +0000 (Tue, 25 Aug 2009) $
  */
 
 /* crypto/buffer/bss_sock.c */
@@ -800,6 +800,9 @@ PSSLContext::PSSLContext(const void * sessionId, PINDEX idSize)
 void PSSLContext::Construct(Method method, const void * sessionId, PINDEX idSize)
 {
   // create the new SSL context
+#if OPENSSL_VERSION_NUMBER >= 0x00909000L
+  const
+#endif
   SSL_METHOD * meth;
 
   switch (method) {
