@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21994 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2009-02-04 22:46:56 +0000 (Wed, 04 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -281,7 +281,7 @@ PObject::Comparison PAbstractList::Compare(const PObject & obj) const
   PAssert(PIsDescendant(&obj, PAbstractList), PInvalidCast);
   Element * elmt1 = info->head;
   Element * elmt2 = ((const PAbstractList &)obj).info->head;
-  while (elmt1 != NULL && elmt2 != NULL) {
+  while (elmt1 != NULL || elmt2 != NULL) {
     if (elmt1 == NULL)
       return LessThan;
     if (elmt2 == NULL)
@@ -1273,6 +1273,7 @@ void PHashTable::CloneContents(const PHashTable * hash)
 
   hashTable = new PHashTable::Table(original->GetSize());
   PAssert(hashTable != NULL, POutOfMemory);
+  hashTable->deleteKeys = original->deleteKeys;
 
   for (PINDEX i = 0; i < sz; i++) {
     Element * lastElement = NULL;
