@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21397 $
+ * $Revision: 23208 $
  * $Author: csoutheren $
- * $Date: 2008-10-21 07:46:34 +0000 (Tue, 21 Oct 2008) $
+ * $Date: 2009-08-12 05:24:02 +0000 (Wed, 12 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -56,11 +56,73 @@ extern "C" int vsprintf(char *, const char *, va_list);
 #define __CLASS__ GetClass()
 
 
+
+///////////////////////////////////////////////////////////////////////////////
+//
+//  Use a custom allocator for PContainer references
+//
+
+template <class Type, class Allocator = std::allocator<Type> >
+class CustomAllocator 
+{
+  public:
+    Type * Allocate(size_t n)
+    { return a.allocate(n); }
+
+    void Deallocate(Type * p, size_t n)
+    { a.deallocate(p, n); }
+
+    Type * Reallocate(Type * p, size_t old_n, size_t new_n)
+    { 
+      if (new_n == old_n)
+        return p;
+
+      Type * np = a.allocate(new_n);
+      memcpy(np, p, (new_n < old_n) ? new_n : old_n );
+      a.deallocate(p, old_n); 
+
+      return np;
+    }
+
+    Allocator a;
+};
+
+#if __GNUCC__
+static CustomAllocator<PContainerReference, __gnu_cxx::pool_allocator> PContainerReference_allocator;
+#else
+static CustomAllocator<PContainerReference> PContainerReference_allocator;
+#endif
+
+static PContainerReference * PContainerReference_new(PINDEX initialSize)
+{
+  PContainerReference * ref = PContainerReference_allocator.Allocate(1);
+  if (ref != NULL) {
+    ref->size = initialSize;
+    ref->count.SetValue(1);
+    ref->deleteObjects = true;
+  }
+  return ref;
+}
+
+static PContainerReference * PContainerReference_new(const PContainerReference & r)
+{
+  PContainerReference * ref = PContainerReference_allocator.Allocate(1);
+  if (ref != NULL) {
+    ref->size = r.size;
+    ref->count.SetValue(1);
+    ref->deleteObjects = r.deleteObjects;
+  }
+  return ref;
+}
+
+static void PContainerReference_delete(PContainerReference * p)
+{ PContainerReference_allocator.Deallocate(p, 1); }
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PContainer::PContainer(PINDEX initialSize)
 {
-  reference = new PContainerReference(initialSize);
+  reference = PContainerReference_new(initialSize);
   PAssert(reference != NULL, POutOfMemory);
 }
 
@@ -72,7 +134,7 @@ PContainer::PContainer(int, const PContainer * cont)
   PAssert(cont != NULL, PInvalidParameter);
   PAssert2(cont->reference != NULL, cont->GetClass(), "Clone of deleted container");
 
-  reference = new PContainerReference(*cont->reference);   // create a new reference
+  reference = PContainerReference_new(*cont->reference);
   PAssert(reference != NULL, POutOfMemory);
 }
 
@@ -104,13 +166,12 @@ void PContainer::AssignContents(const PContainer & cont)
 
   if (--reference->count == 0) {
     DestroyContents();
-    delete reference;
+    PContainerReference_delete(reference);
     reference = NULL;
   }
 
   ++cont.reference->count;
   reference = cont.reference;
-
 }
 
 
@@ -123,7 +184,7 @@ void PContainer::Destruct()
     
     else {
       DestroyContents();
-      delete reference;
+      PContainerReference_delete(reference);
       reference = NULL;
     }
   }
@@ -147,7 +208,7 @@ PBoolean PContainer::MakeUnique()
     return PTrue;
 
   PContainerReference * oldReference = reference;
-  reference = new PContainerReference(*reference);
+  reference = PContainerReference_new(*reference);
   --oldReference->count;
 
   return PFalse;
@@ -156,6 +217,14 @@ PBoolean PContainer::MakeUnique()
 
 ///////////////////////////////////////////////////////////////////////////////
 
+#if __GNUCC__
+static CustomAllocator<char, __gnu_cxx::mt_allocator> PAbstractArray_allocator;
+#else
+static CustomAllocator<char> PAbstractArray_allocator;
+#endif
+
+
+
 PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes, PINDEX initialSize)
   : PContainer(initialSize)
 {
@@ -165,8 +234,10 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes, PINDEX initialSize)
   if (GetSize() == 0)
     theArray = NULL;
   else {
-    theArray = (char *)calloc(GetSize(), elementSize);
+    //theArray = (char *)calloc(GetSize(), elementSize);
+    theArray = PAbstractArray_allocator.Allocate(GetSize() * elementSize);
     PAssert(theArray != NULL, POutOfMemory);
+    memset(theArray, 0, GetSize() * elementSize);
   }
 
   allocatedDynamically = PTrue;
@@ -188,7 +259,7 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes,
     theArray = NULL;
   else if (dynamicAllocation) {
     PINDEX sizebytes = elementSize*GetSize();
-    theArray = (char *)malloc(sizebytes);
+    theArray = PAbstractArray_allocator.Allocate(sizebytes);
     PAssert(theArray != NULL, POutOfMemory);
     memcpy(theArray, PAssertNULL(buffer), sizebytes);
   }
@@ -200,8 +271,10 @@ PAbstractArray::PAbstractArray(PINDEX elementSizeInBytes,
 void PAbstractArray::DestroyContents()
 {
   if (theArray != NULL) {
-    if (allocatedDynamically)
-      free(theArray);
+    if (allocatedDynamically) {
+      PAbstractArray_allocator.Deallocate(theArray, elementSize*GetSize());
+      //free(theArray);
+    }
     theArray = NULL;
   }
 }
@@ -219,7 +292,7 @@ void PAbstractArray::CloneContents(const PAbstractArray * array)
 {
   elementSize = array->elementSize;
   PINDEX sizebytes = elementSize*GetSize();
-  char * newArray = (char *)malloc(sizebytes);
+  char * newArray = PAbstractArray_allocator.Allocate(sizebytes);
   if (newArray == NULL)
     reference->size = 0;
   else
@@ -316,7 +389,7 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
     if (newsizebytes == 0)
       newArray = NULL;
     else {
-      if ((newArray = (char *)malloc(newsizebytes)) == NULL)
+      if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
         return PFalse;
   
       if (theArray != NULL)
@@ -324,29 +397,29 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
     }
 
     --reference->count;
-    reference = new PContainerReference(newSize);
+    reference = PContainerReference_new(newSize);
 
   } else {
 
     if (theArray != NULL) {
       if (newsizebytes == 0) {
         if (allocatedDynamically)
-          free(theArray);
+          PAbstractArray_allocator.Deallocate(theArray, oldsizebytes);
         newArray = NULL;
       }
       else if (allocatedDynamically) {
-        if ((newArray = (char *)realloc(theArray, newsizebytes)) == NULL)
+        if ((newArray = (char *)PAbstractArray_allocator.Reallocate(theArray, oldsizebytes, newsizebytes)) == NULL)
           return PFalse;
       }
       else {
-        if ((newArray = (char *)malloc(newsizebytes)) == NULL)
+        if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
           return PFalse;
         memcpy(newArray, theArray, PMIN(newsizebytes, oldsizebytes));
         allocatedDynamically = PTrue;
       }
     }
     else if (newsizebytes != 0) {
-      if ((newArray = (char *)malloc(newsizebytes)) == NULL)
+      if ((newArray = PAbstractArray_allocator.Allocate(newsizebytes)) == NULL)
         return PFalse;
     }
     else
@@ -365,7 +438,7 @@ PBoolean PAbstractArray::InternalSetSize(PINDEX newSize, PBoolean force)
 void PAbstractArray::Attach(const void *buffer, PINDEX bufferSize)
 {
   if (allocatedDynamically && theArray != NULL)
-    free(theArray);
+    PAbstractArray_allocator.Deallocate(theArray, elementSize*GetSize());
 
   theArray = (char *)buffer;
   reference->size = bufferSize;
@@ -1862,11 +1935,13 @@ PString & PString::vsprintf(const char * fmt, va_list arg)
   PAssert(SetSize(2000), POutOfMemory);
   ::vsprintf(theArray+len, fmt, arg);
 #else
-  PINDEX size = len;
+  PINDEX providedSpace = 0;
+  int requiredSpace;
   do {
-    size += 1000;
-    PAssert(SetSize(size), POutOfMemory);
-  } while (_vsnprintf(theArray+len, size-len, fmt, arg) == -1);
+    providedSpace += 1000;
+    PAssert(SetSize(providedSpace+len), POutOfMemory);
+    requiredSpace = _vsnprintf(theArray+len, providedSpace, fmt, arg);
+  } while (requiredSpace == -1 || requiredSpace >= providedSpace);
 #endif // P_VXWORKS
 
   PAssert(MakeMinimumSize(), POutOfMemory);
