@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22040 $
+ * $Revision: 22392 $
  * $Author: rjongbloed $
- * $Date: 2009-02-11 23:08:26 +0000 (Wed, 11 Feb 2009) $
+ * $Date: 2009-04-07 04:43:05 +0000 (Tue, 07 Apr 2009) $
  */
 
 #include <ptlib.h>
@@ -545,7 +545,9 @@ PString PTime::AsString(const char * format, int zone) const
 
 extern "C" {
 
-#ifndef STDAPICALLTYPE
+#ifdef _WIN32
+#define STDAPICALLTYPE __stdcall
+#else
 #define STDAPICALLTYPE
 #endif
 
