@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23018 $
+ * $Revision: 23177 $
  * $Author: rjongbloed $
- * $Date: 2009-07-01 00:13:04 +0000 (Wed, 01 Jul 2009) $
+ * $Date: 2009-07-29 07:46:33 +0000 (Wed, 29 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -67,8 +67,14 @@ void CALLBACK CompletionRoutine(DWORD dwError,
 #endif // P_QOS
 
 
-#if P_HAS_IPV6 || defined(AI_NUMERICHOST)
-  #define HAS_GETADDRINFO 1
+#if !defined(P_MINGW) && !defined(P_CYGWIN)
+  #if P_HAS_IPV6 || defined(AI_NUMERICHOST)
+    #define HAS_GETADDRINFO 1
+  #endif
+#else
+  #if WINVER > 0x500
+    #define HAS_GETADDRINFO 1
+  #endif
 #endif
 
 
