@@ -23,9 +23,9 @@
  *
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
- * $Revision: 20385 $
+ * $Revision: 23298 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2009-08-28 08:53:17 +0000 (Fri, 28 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -39,11 +39,9 @@
 #include <ptlib/videoio.h>
 #include <ptclib/delaychan.h>
 
-#ifdef __MACOSX__
-namespace PWLibStupidOSXHacks {
+namespace PWLibStupidLinkerHacks {
   int loadFakeVideoStuff;
 };
-#endif
 
 enum {
   eMovingBlocks,
@@ -2146,8 +2144,6 @@ void PVideoTools::FillYUV420Rect(BYTE * frame, unsigned frameWidth, unsigned fra
                    int rectWidth, int rectHeight,
                    int r, int g,  int b)
 {
-// PTRACE(0,"x,y is"<<xPos<<" "<<yPos<<" and size is "<<rectWidth<<" "<<rectHeight);
-
   //This routine fills a region of the video image with data. It is used as the central
   //point because one only has to add other image formats here.
 
