@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 22240 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2009-03-23 00:54:33 +0000 (Mon, 23 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -73,6 +73,12 @@ typedef enum _NDIS_MEDIUM {
 #define USE_VPACKET
 #include <ptlib/msos/ptlib/epacket.h>
 
+#define IPv6_ENABLED (P_HAS_IPV6 && (WINVER>=0x501))
+
+#if IPv6_ENABLED
+#include <ptlib/msos/ptlib/addrv6.h>
+#endif
+
 #ifdef USE_VPACKET
 #define PACKET_SERVICE_NAME "Packet"
 #define PACKET_VXD_NAME     "VPacket"
@@ -1328,8 +1334,10 @@ public:
     }
   }
 
-  const MIB_IPFORWARDTABLE * operator->() const { return  (const MIB_IPFORWARDTABLE *)(const BYTE *)buffer; }
-  const MIB_IPFORWARDTABLE & operator *() const { return *(const MIB_IPFORWARDTABLE *)(const BYTE *)buffer; }
+  const MIB_IPFORWARDTABLE * Ptr() const { return (const MIB_IPFORWARDTABLE *)(const BYTE *)buffer; }
+  const MIB_IPFORWARDTABLE * operator->() const { return  Ptr(); }
+  const MIB_IPFORWARDTABLE & operator *() const { return *Ptr(); }
+  operator const MIB_IPFORWARDTABLE *  () const { return  Ptr(); }
 
   private:
     PBYTEArray buffer;
@@ -1351,18 +1359,140 @@ public:
     }
   }
 
-  const MIB_IPADDRTABLE * operator->() const { return  (const MIB_IPADDRTABLE *)(const BYTE *)buffer; }
-  const MIB_IPADDRTABLE & operator *() const { return *(const MIB_IPADDRTABLE *)(const BYTE *)buffer; }
+  const MIB_IPADDRTABLE * Ptr() const { return (const MIB_IPADDRTABLE *)(const BYTE *)buffer; }
+  const MIB_IPADDRTABLE * operator->() const { return  Ptr(); }
+  const MIB_IPADDRTABLE & operator *() const { return *Ptr(); }
+  operator const MIB_IPADDRTABLE *  () const { return  Ptr(); }
 
   private:
     PBYTEArray buffer;
 };
 
 
+#if IPv6_ENABLED
+class PIPAdaptersAddressTable
+{
+public:
+
+  PIPAdaptersAddressTable(DWORD dwFlags = GAA_FLAG_INCLUDE_PREFIX
+                                        | GAA_FLAG_SKIP_ANYCAST
+                                        | GAA_FLAG_SKIP_DNS_SERVER
+                                        | GAA_FLAG_SKIP_MULTICAST) 
+  {
+    ULONG size = 0;
+    DWORD error = GetAdaptersAddresses(AF_UNSPEC, dwFlags, NULL, NULL, &size);
+    if (buffer.SetSize(size))
+      error = GetAdaptersAddresses(AF_UNSPEC, dwFlags, NULL, (IP_ADAPTER_ADDRESSES *)buffer.GetPointer(), &size);
+
+    if (error != NO_ERROR) {
+      buffer.SetSize(0);
+      buffer.SetSize(sizeof(IP_ADAPTER_ADDRESSES)); // So ->NumAdapters returns zero
+    }
+  }
+
+  const IP_ADAPTER_ADDRESSES * Ptr() const { return  (const IP_ADAPTER_ADDRESSES *)(const BYTE *)buffer; }
+  const IP_ADAPTER_ADDRESSES * operator->() const { return  Ptr(); }
+  const IP_ADAPTER_ADDRESSES & operator *() const { return *Ptr(); }
+  operator const IP_ADAPTER_ADDRESSES *  () const { return  Ptr(); }
+
+private:
+  PBYTEArray buffer;
+};
+
+
+#include <tchar.h>
+
+class PIPRouteTableIPv6 : public PIPRouteTable
+{
+public:
+
+  PIPRouteTableIPv6()
+  {
+    buffer.SetSize(sizeof(MIB_IPFORWARD_TABLE2)); // So ->NumEntries returns zero
+
+    HINSTANCE hInst = LoadLibrary(_T("iphlpapi.dll"));
+    if (hInst != NULL) {
+      GETIPFORWARDTABLE2 pfGetIpForwardTable2 = (GETIPFORWARDTABLE2)GetProcAddress(hInst, _T("GetIpForwardTable2"));
+      FREEMIBTABLE pfFreeMibTable = (FREEMIBTABLE)GetProcAddress(hInst, _T("FreeMibTable"));
+      if (pfGetIpForwardTable2 != NULL && pfFreeMibTable != NULL) {
+        PMIB_IPFORWARD_TABLE2 pt = NULL;
+        DWORD dwError = (*pfGetIpForwardTable2)(AF_UNSPEC, &pt);
+        if (dwError == NO_ERROR) {
+          buffer.SetSize(pt->NumEntries * sizeof(MIB_IPFORWARD_ROW2));
+          memcpy(buffer.GetPointer(), pt, buffer.GetSize());
+          (*pfFreeMibTable)(pt);
+        }
+      }
+      FreeLibrary(hInst);
+    }
+  }
+
+  const MIB_IPFORWARD_TABLE2 * Ptr() const { return  (const MIB_IPFORWARD_TABLE2 *)(const BYTE *)buffer; }
+  const MIB_IPFORWARD_TABLE2 * operator->() const { return  Ptr(); }
+  const MIB_IPFORWARD_TABLE2 & operator *() const { return *Ptr(); }
+  operator const MIB_IPFORWARD_TABLE2 *  () const { return  Ptr(); }
+
+
+  bool ValidateAddress(DWORD ifIndex, LPSOCKADDR lpSockAddr)
+  {
+    int numEntries = Ptr()->NumEntries;
+    if (numEntries == 0)
+      return true;
+
+    if (lpSockAddr == NULL)
+      return false;
+
+    const MIB_IPFORWARD_ROW2 * row = Ptr()->Table;
+    for (int i = 0; i < numEntries; i++, row++) {
+      if (row->InterfaceIndex == ifIndex &&
+          row->DestinationPrefix.Prefix.si_family == lpSockAddr->sa_family) {
+        switch (lpSockAddr->sa_family) {
+          case AF_INET :
+            if (row->DestinationPrefix.Prefix.Ipv4.sin_addr.S_un.S_addr == ((sockaddr_in *)lpSockAddr)->sin_addr.S_un.S_addr)
+              return true;
+            break;
+
+          case AF_INET6 :
+            if (memcmp(row->DestinationPrefix.Prefix.Ipv6.sin6_addr.u.Byte,
+                ((sockaddr_in6 *)lpSockAddr)->sin6_addr.u.Byte, sizeof(in6_addr)) == 0)
+              return true;
+        }
+      }
+    }
+
+    return false;
+  }
+
+private:
+  PBYTEArray buffer;
+};
+#endif // IPv6_ENABLED
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
-PBoolean PIPSocket::GetGatewayAddress(Address & addr)
+PBoolean PIPSocket::GetGatewayAddress(Address & addr, int version)
 {
+  if (version == 6) {
+#if IPv6_ENABLED
+    PIPRouteTableIPv6 routes;
+    int numEntries = routes->NumEntries;
+    if (numEntries > 0) {
+      const MIB_IPFORWARD_ROW2 * row = routes->Table;
+      for (int i = 0; i < numEntries; i++, row++) {
+        if (row->NextHop.si_family == AF_INET6) {
+          PIPSocket::Address hop(row->NextHop.Ipv6.sin6_addr);
+          if (hop.IsValid()) {
+            addr = hop;
+            return true;
+          }
+        }
+      }
+    }
+#endif
+    return false;
+  }
+
   PIPRouteTable routes;
   for (unsigned i = 0; i < routes->dwNumEntries; ++i) {
     if (routes->table[i].dwForwardMask == 0) {
@@ -1374,8 +1504,28 @@ PBoolean PIPSocket::GetGatewayAddress(Address & addr)
 }
 
 
-PString PIPSocket::GetGatewayInterface()
+PString PIPSocket::GetGatewayInterface(int version)
 {
+  if (version == 6) {
+#if IPv6_ENABLED
+    PIPRouteTableIPv6 routes;
+    int numEntries = routes->NumEntries;
+    if (numEntries > 0) {
+      const MIB_IPFORWARD_ROW2 * row = routes->Table;
+      for (int i = 0; i < numEntries; i++, row++) {
+        if (row->NextHop.si_family == AF_INET6 && PIPSocket::Address(row->NextHop.Ipv6.sin6_addr).IsValid()) {
+          PIPAdaptersAddressTable adapters;
+          for (const IP_ADAPTER_ADDRESSES * adapter = &*adapters; adapter != NULL; adapter = adapter->Next) {
+            if (adapter->IfIndex == row->InterfaceIndex)
+              return adapter->Description;
+          }
+        }
+      }
+    }
+#endif
+    return PString::Empty();
+  }
+
   PIPRouteTable routes;
   for (unsigned i = 0; i < routes->dwNumEntries; ++i) {
     if (routes->table[i].dwForwardMask == 0) {
@@ -1388,8 +1538,36 @@ PString PIPSocket::GetGatewayInterface()
   return PString::Empty();
 } 
 
-PIPSocket::Address PIPSocket::GetGatewayInterfaceAddress()
+
+PIPSocket::Address PIPSocket::GetGatewayInterfaceAddress(int version)
 {
+#if IPv6_ENABLED
+  if (version == 6) {
+    PIPRouteTableIPv6 routes;
+    if (routes->NumEntries > 0) {
+      PIPAdaptersAddressTable interfaces;
+
+      for (const IP_ADAPTER_ADDRESSES * adapter = &*interfaces; adapter != NULL; adapter = adapter->Next) {
+        for (PIP_ADAPTER_UNICAST_ADDRESS unicast = adapter->FirstUnicastAddress; unicast != NULL; unicast = unicast->Next) {
+          if (unicast->Address.lpSockaddr->sa_family != AF_INET6)
+            continue;
+
+          PIPSocket::Address ip(((sockaddr_in6 *)unicast->Address.lpSockaddr)->sin6_addr);
+
+          if (routes->NumEntries == 0)
+            return ip;
+
+          // Check if local-link addresses supported (scope_id != 0)
+          if ((PIPSocket::GetDefaultV6ScopeId() || (ip[0] != 0xFE && ip[1] != 0x80))
+              &&
+              routes.ValidateAddress(adapter->IfIndex, unicast->Address.lpSockaddr))
+            return ip;
+        }
+      }
+    }
+  }
+#endif
+
   PIPRouteTable routes;
   for (unsigned i = 0; i < routes->dwNumEntries; ++i) {
     if (routes->table[i].dwForwardMask == 0) {
@@ -1432,6 +1610,61 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 }
 
 
+#ifdef _MSC_VER
+#pragma optimize("g", off)
+#endif
+
+bool PIPSocket::WaitForRouteTableChange(const PTimeInterval & timeout, PSyncPoint * cancellation)
+{
+  HANDLE handle = NULL;
+  OVERLAPPED overlap;
+  memset(&overlap, 0, sizeof(overlap));
+
+  // Do this so compatible with older operating systems
+  PDynaLink dll("iphlpapi.dll");
+  BOOL (*pCancelIPChangeNotify)(LPOVERLAPPED) = NULL;
+  bool doNotify = dll.GetFunction("CancelIPChangeNotify", (PDynaLink::Function&)pCancelIPChangeNotify);
+
+  if (doNotify) {
+    DWORD error = NotifyAddrChange(&handle, &overlap);
+    if (error != ERROR_IO_PENDING) {
+      PTRACE(1, "PTlib\tCould not get network interface change notification: error=" << error);
+      doNotify = false;
+    }
+  }
+
+  if (!doNotify) {
+    if (cancellation != NULL)
+      return cancellation->Wait(timeout);
+
+    PThread::Sleep(timeout);
+    return false;
+  }
+
+  if (cancellation == NULL)
+    return WaitForSingleObject(handle, timeout.GetInterval()) == WAIT_OBJECT_0;
+
+  HANDLE handles[2];
+  handles[0] = handle;
+  handles[1] = cancellation->GetHandle();
+  switch (WaitForMultipleObjects(2, handles, false, timeout.GetInterval())) {
+    case WAIT_OBJECT_0 :
+      return true;
+
+    case WAIT_OBJECT_0+1 :
+      pCancelIPChangeNotify(&overlap);
+      // Do next case
+
+    default :
+      return false;
+  }
+}
+
+#ifdef _MSC_VER
+#pragma optimize("", on)
+#endif
+
+
 PIPSocket::Address PIPSocket::GetRouteAddress(PIPSocket::Address remoteAddress)
 {
   DWORD best;
@@ -1455,7 +1688,6 @@ PBoolean PIPSocket::IsAddressReachable(PIPSocket::Address localIP,
                                        PIPSocket::Address localMask, 
                                        PIPSocket::Address remoteIP)
 {
-
   BYTE t = 255;
   int t1=t,t2=t,t3 =t,t4=t;
   int b1=0,b2=0,b3=0,b4=0;
@@ -1505,6 +1737,57 @@ PString PIPSocket::GetInterface(PIPSocket::Address addr)
 
 PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & table, PBoolean includeDown)
 {
+#if IPv6_ENABLED
+  // Adding IPv6 addresses
+  PIPRouteTableIPv6 routes;
+  PIPAdaptersAddressTable interfaces;
+  PIPInterfaceAddressTable byAddress;
+
+  PINDEX count = 0; // address count
+
+  if (!table.SetSize(0))
+    return false;
+
+  for (const IP_ADAPTER_ADDRESSES * adapter = &*interfaces; adapter != NULL; adapter = adapter->Next) {
+    if (!includeDown && (adapter->OperStatus != IfOperStatusUp))
+      continue;
+
+    for (PIP_ADAPTER_UNICAST_ADDRESS unicast = adapter->FirstUnicastAddress; unicast != NULL; unicast = unicast->Next) {
+      if (!routes.ValidateAddress(adapter->IfIndex, unicast->Address.lpSockaddr))
+        continue;
+
+      PStringStream macAddr;
+      macAddr << ::hex << ::setfill('0');
+      for (unsigned b = 0; b < adapter->PhysicalAddressLength; ++b)
+        macAddr << setw(2) << (unsigned)adapter->PhysicalAddress[b];
+
+      if (unicast->Address.lpSockaddr->sa_family == AF_INET && PIPSocket::GetDefaultIpAddressFamily() == AF_INET) {
+        PIPSocket::Address ip(((sockaddr_in *)unicast->Address.lpSockaddr)->sin_addr);
+
+        // Find out address index in byAddress table for the mask
+        DWORD dwMask = 0L;
+        for (unsigned i = 0; i < byAddress->dwNumEntries; ++i) {
+          if (adapter->IfIndex == byAddress->table[i].dwIndex) {
+            dwMask = byAddress->table[i].dwMask;
+            break;
+          }
+        } // find mask for the address
+
+        table.SetAt(count++, new InterfaceEntry(adapter->Description, ip, dwMask, macAddr));
+
+      } // ipv4
+      else if (unicast->Address.lpSockaddr->sa_family == AF_INET6 && PIPSocket::GetDefaultIpAddressFamily() == AF_INET6) {
+        PIPSocket::Address ip(((sockaddr_in6 *)unicast->Address.lpSockaddr)->sin6_addr);
+
+        // Check if local-link addresses supported (scope_id != 0)
+        if (PIPSocket::GetDefaultV6ScopeId() || (ip[0] != 0xFE && ip[1] != 0x80))
+          table.SetAt(count++, new InterfaceEntry(adapter->Description, ip, 0L, macAddr));
+      } // ipv6
+    }
+  }
+
+#else
+
   PIPInterfaceAddressTable byAddress;
 
   if (!table.SetSize(byAddress->dwNumEntries))
@@ -1533,6 +1816,9 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & table, PBoolean includeDo
   }
 
   table.SetSize(count); // May shrink due to "down" interfaces.
+
+#endif
+
   return true;
 }
 
