@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21176 $
- * $Author: ms30002000 $
- * $Date: 2008-09-27 20:34:32 +0000 (Sat, 27 Sep 2008) $
+ * $Revision: 23291 $
+ * $Author: rjongbloed $
+ * $Date: 2009-08-28 06:07:44 +0000 (Fri, 28 Aug 2009) $
  */
 
 #include <ptlib.h>
@@ -48,11 +48,9 @@
 #include <ptlib/wm/mmsystemx.h>
 #endif
 
-#if defined(P_MINGW)
 namespace PWLibStupidLinkerHacks {
    int loadWindowsMultimediaStuff;
 };
-#endif
      
 class PSound;
 
@@ -611,6 +609,7 @@ PSoundChannelWin32::PSoundChannelWin32(const PString & device,
 
 void PSoundChannelWin32::Construct()
 {
+  opened = false;
   direction = Player;
   hWaveOut = NULL;
   hWaveIn = NULL;
@@ -639,29 +638,35 @@ PString PSoundChannelWin32::GetName() const
 }
 
 
-static PString GetWaveOutDeviceName(UINT id)
+static bool GetWaveOutDeviceName(UINT id, PString & name)
 {
-  if (id == WAVE_MAPPER)
-    return "Default";
+  if (id == WAVE_MAPPER) {
+    name = "Default";
+    return true;
+  }
 
-  PString str;
   WAVEOUTCAPS caps;
-  if (waveOutGetDevCaps(id, &caps, sizeof(caps)) == 0)
-    str = caps.szPname;
-  return str.Trim();
+  if (waveOutGetDevCaps(id, &caps, sizeof(caps)) != 0)
+    return false;
+
+  name = PString(caps.szPname).Trim();
+  return true;
 }
 
 
-static PCaselessString GetWaveInDeviceName(UINT id)
+static bool GetWaveInDeviceName(UINT id, PString & name)
 {
-  if (id == WAVE_MAPPER)
-    return "Default";
+  if (id == WAVE_MAPPER) {
+    name = "Default";
+    return true;
+  }
 
-  PString str;
   WAVEINCAPS caps;
-  if (waveInGetDevCaps(id, &caps, sizeof(caps)) == 0)
-    str = caps.szPname;
-  return str.Trim();
+  if (waveInGetDevCaps(id, &caps, sizeof(caps)) != 0)
+    return false;
+
+  name = PString(caps.szPname).Trim();
+  return true;
 }
 
 
@@ -676,14 +681,18 @@ PStringArray PSoundChannelWin32::GetDeviceNames(Directions dir)
     case Player :
       numDevs = waveOutGetNumDevs();
       do {
-        devices.AppendString(GetWaveOutDeviceName(id));
+        PCaselessString dev;
+        if (GetWaveOutDeviceName(id, dev))
+          devices.AppendString(dev);
       } while (++id < numDevs);
       break;
 
     case Recorder :
       numDevs = waveInGetNumDevs();
       do {
-        devices.AppendString(GetWaveInDeviceName(id));
+        PCaselessString dev;
+        if (GetWaveInDeviceName(id, dev))
+          devices.AppendString(dev);
       } while (++id < numDevs);
       break;
   }
@@ -705,12 +714,12 @@ PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir,
     switch (dir) {
       case Player :
         if (id < waveOutGetNumDevs())
-          deviceName = GetWaveOutDeviceName(id);
+          GetWaveOutDeviceName(id, deviceName);
         break;
 
       case Recorder :
         if (id < waveInGetNumDevs())
-          deviceName = GetWaveInDeviceName(id);
+          GetWaveInDeviceName(id, deviceName);
         break;
     }
   }
@@ -721,8 +730,8 @@ PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir,
       case Player :
         numDevs = waveOutGetNumDevs();
         do {
-          PCaselessString str = GetWaveOutDeviceName(id);
-          if (str == device.Mid(offset)) {
+          PCaselessString str;
+          if (GetWaveOutDeviceName(id, str) && str == device.Mid(offset)) {
             deviceName = str;
             break;
           }
@@ -732,8 +741,8 @@ PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir,
       case Recorder :
         numDevs = waveInGetNumDevs();
         do {
-          PCaselessString str = GetWaveInDeviceName(id);
-          if (str == device.Mid(offset)) {
+          PCaselessString str;
+          if (GetWaveInDeviceName(id, str) && str == device.Mid(offset)) {
             deviceName = str;
             break;
           }
@@ -842,13 +851,14 @@ PBoolean PSoundChannelWin32::OpenDevice(unsigned id)
     }
   }
 
+  opened = true;
   os_handle = id;
   return PTrue;
 }
 
 PBoolean PSoundChannelWin32::IsOpen() const
 { 
-  return os_handle >= 0;
+  return opened ? PTrue : PFalse;
 }
 
 PBoolean PSoundChannelWin32::SetFormat(unsigned numChannels,
@@ -917,6 +927,7 @@ PBoolean PSoundChannelWin32::Close()
     hMixer = NULL;
   }
 
+  opened = false;
   os_handle = -1;
   return PTrue;
 }
@@ -928,7 +939,7 @@ PBoolean PSoundChannelWin32::SetBuffers(PINDEX size, PINDEX count)
 
   PAssert(size > 0 && count > 0, PInvalidParameter);
 
-  PTRACE(1, "WinSnd\tSetting sounds buffers to " << count << " x " << size);
+  PTRACE(3, "WinSnd\tSetting sounds buffers to " << count << " x " << size);
 
   PBoolean ok = PTrue;
 
@@ -1446,7 +1457,7 @@ PBoolean PSoundChannelWin32::GetVolume(unsigned & oldVolume)
   details.cbDetails = sizeof(volume);
   details.paDetails = &volume;
 
-  MMRESULT result = mixerSetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE);
+  MMRESULT result = mixerGetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE);
   if (result != MMSYSERR_NOERROR)
     return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);
 
