@@ -24,9 +24,9 @@
  * Contributor(s): ______________________________________.
  *                 Walter H Whitlock (twohives@nc.rr.com)
  *
- * $Revision: 20834 $
- * $Author: rjongbloed $
- * $Date: 2008-09-03 04:00:55 +0000 (Wed, 03 Sep 2008) $
+ * $Revision: 22039 $
+ * $Author: dsandras $
+ * $Date: 2009-02-11 19:07:07 +0000 (Wed, 11 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -38,105 +38,25 @@
 #include <ptlib/pluginmgr.h>
 #include <ptclib/delaychan.h>
 
+#if P_VFW_CAPTURE
+
 #ifdef _MSC_VER
-#ifndef _WIN32_WCE
 #pragma comment(lib, "vfw32.lib")
 #pragma comment(lib, "gdi32.lib")
 #endif
-#endif
+
+namespace PWLibStupidLinkerHacks {
+   int loadVideoForWindowsStuff;
+};
+    
+#include <vfw.h>
 
 #ifdef __MINGW32__
 
 #define VHDR_DONE       0x00000001
 #define VHDR_KEYFRAME   0x00000008
-
-typedef struct videohdr_tag {
-    LPBYTE      lpData;
-    DWORD       dwBufferLength;
-    DWORD       dwBytesUsed;
-    DWORD       dwTimeCaptured;
-    DWORD       dwUser;
-    DWORD       dwFlags;
-    DWORD       dwReserved[4];
-} *LPVIDEOHDR;
-
-typedef struct tagCapDriverCaps {
-    UINT        wDeviceIndex;
-    BOOL        fHasOverlay;
-    BOOL        fHasDlgVideoSource;
-    BOOL        fHasDlgVideoFormat;
-    BOOL        fHasDlgVideoDisplay;
-    BOOL        fCaptureInitialized;
-    BOOL        fDriverSuppliesPalettes;
-    HANDLE      hVideoIn;
-    HANDLE      hVideoOut;
-    HANDLE      hVideoExtIn;
-    HANDLE      hVideoExtOut;
-} CAPDRIVERCAPS, *LPCAPDRIVERCAPS;
-
-typedef struct tagCaptureParms {
-    DWORD       dwRequestMicroSecPerFrame;  // Requested capture rate
-    BOOL        fMakeUserHitOKToCapture;    // Show "Hit OK to cap" dlg?
-    WORD        wPercentDropForError;       // Give error msg if > (10%)
-    BOOL        fYield;                     // Capture via background task?
-    DWORD       dwIndexSize;                // Max index size in frames (32K)
-    WORD        wChunkGranularity;          // Junk chunk granularity (2K)
-    BOOL        fUsingDOSMemory;            // Use DOS buffers?
-    WORD        wNumVideoRequested;         // # video buffers, If 0, autocalc
-    BOOL        fCaptureAudio;              // Capture audio?
-    WORD        wNumAudioRequested;         // # audio buffers, If 0, autocalc
-    WORD        vKeyAbort;                  // Virtual key causing abort
-    BOOL        fAbortLeftMouse;            // Abort on left mouse?
-    BOOL        fAbortRightMouse;           // Abort on right mouse?
-    BOOL        fLimitEnabled;              // Use wTimeLimit?
-    WORD        wTimeLimit;                 // Seconds to capture
-    BOOL        fMCIControl;                // Use MCI video source?
-    BOOL        fStepMCIDevice;             // Step MCI device?
-    DWORD       dwMCIStartTime;             // Time to start in MS
-    DWORD       dwMCIStopTime;              // Time to stop in MS
-    BOOL        fStepCaptureAt2x;           // Perform spatial averaging 2x
-    WORD        wStepCaptureAverageFrames;  // Temporal average n Frames
-    DWORD       dwAudioBufferSize;          // Size of audio bufs (0 = default)
-    BOOL        fDisableWriteCache;         // Attempt to disable write cache
-} CAPTUREPARMS, FAR *LPCAPTUREPARMS;
-
-typedef struct tagCapStatus {
-    UINT        uiImageWidth;               // Width of the image
-    UINT        uiImageHeight;              // Height of the image
-    BOOL        fLiveWindow;                // Now Previewing video?
-    BOOL        fOverlayWindow;             // Now Overlaying video?
-    BOOL        fScale;                     // Scale image to client?
-    POINT       ptScroll;                   // Scroll position
-    BOOL        fUsingDefaultPalette;       // Using default driver palette?
-    BOOL        fAudioHardware;             // Audio hardware present?
-    BOOL        fCapFileExists;             // Does capture file exist?
-    DWORD       dwCurrentVideoFrame;        // # of video frames cap'td
-    DWORD       dwCurrentVideoFramesDropped;// # of video frames dropped
-    DWORD       dwCurrentWaveSamples;       // # of wave samples cap'td
-    DWORD       dwCurrentTimeElapsedMS;     // Elapsed capture duration
-    HPALETTE    hPalCurrent;                // Current palette in use
-    BOOL        fCapturingNow;              // Capture in progress?
-    DWORD       dwReturn;                   // Error value after any operation
-    WORD        wNumVideoAllocated;         // Actual number of video buffers
-    WORD        wNumAudioAllocated;         // Actual number of audio buffers
-} CAPSTATUS, FAR *LPCAPSTATUS;
-
-#define WM_CAP_START                    WM_USER
-#define WM_CAP_SET_CALLBACK_ERROR       (WM_CAP_START+  2)
-#define WM_CAP_SET_CALLBACK_FRAME       (WM_CAP_START+  5)
-#define WM_CAP_SET_CALLBACK_VIDEOSTREAM (WM_CAP_START+  6)
-#define WM_CAP_GET_USER_DATA            (WM_CAP_START+  8)
-#define WM_CAP_SET_USER_DATA            (WM_CAP_START+  9)
-#define WM_CAP_DRIVER_CONNECT           (WM_CAP_START+  10)
-#define WM_CAP_DRIVER_DISCONNECT        (WM_CAP_START+  11)
-#define WM_CAP_DRIVER_GET_CAPS          (WM_CAP_START+  14)
-#define WM_CAP_GET_VIDEOFORMAT          (WM_CAP_START+  44)
-#define WM_CAP_SET_VIDEOFORMAT          (WM_CAP_START+  45)
-#define WM_CAP_SET_PREVIEW              (WM_CAP_START+  50)
-#define WM_CAP_GET_STATUS               (WM_CAP_START+  54)
-#define WM_CAP_GRAB_FRAME_NOSTOP        (WM_CAP_START+  61)
-#define WM_CAP_SET_SEQUENCE_SETUP       (WM_CAP_START+  64)
-#define WM_CAP_GET_SEQUENCE_SETUP       (WM_CAP_START+  65)
+#define WM_USER 1024
+#define WM_CAP_FIRSTA (WM_USER)
 
 #define capSetCallbackOnError(hwnd, fpProc)        ((BOOL)::SendMessage(hwnd, WM_CAP_SET_CALLBACK_ERROR, 0, (LPARAM)(LPVOID)(fpProc)))
 #define capSetCallbackOnFrame(hwnd, fpProc)        ((BOOL)::SendMessage(hwnd, WM_CAP_SET_CALLBACK_FRAME, 0, (LPARAM)(LPVOID)(fpProc)))
@@ -156,20 +76,15 @@ typedef struct tagCapStatus {
 #define capCaptureGetSetup(hwnd, s, wSize)         ((BOOL)::SendMessage(hwnd, WM_CAP_GET_SEQUENCE_SETUP, (WPARAM)(wSize), (LPARAM)(LPVOID)(LPCAPTUREPARMS)(s)))
 
 extern "C" {
-HWND VFWAPI capCreateCaptureWindowA (LPCSTR lpszWindowName, DWORD dwStyle,
-             int x, int y, int nWidth, int nHeight,
-             HWND hwndParent, int nID);
 #ifdef _MSC_VER
 BOOL VFWAPI capGetDriverDescriptionA (WORD wDriverIndex, LPSTR lpszName,
               int cbName, LPSTR lpszVer, int cbVer);
 #endif
 }
 
-#define capGetDriverDescription capGetDriverDescriptionA
-#define capCreateCaptureWindow capCreateCaptureWindowA
-
 #endif // __MINGW32
 
+
 #define STEP_GRAB_CAPTURE 1
 
 
@@ -839,8 +754,10 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameData(BYTE * buffer, PINDEX *
 
 PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, PINDEX * bytesReturned)
 {
-  if (!frameAvailable.Wait(1000))
+  if (!frameAvailable.Wait(1000)) {
+    PTRACE(1, "PVidInp\tTimeout waiting for frame grab!");
     return PFalse;
+  }
 
   bool retval = false;
 
@@ -1004,8 +921,8 @@ PBoolean PVideoInputDevice_VideoForWindows::InitialiseCapture()
   capPreview(hCaptureWindow, PFalse);
 
 #if PTRACING
-  if (PTrace::CanTrace(6))
-    TestAllFormats(); // list acceptable formats and frame resolutions for video capture driver
+  //if (PTrace::CanTrace(6))
+  //  TestAllFormats(); // list acceptable formats and frame resolutions for video capture driver
 #endif
   
   return SetFrameRate(frameRate) && SetColourFormatConverter(colourFormat.IsEmpty() ? PString("YUV420P") : colourFormat);
@@ -1040,6 +957,8 @@ void PVideoInputDevice_VideoForWindows::HandleCapture(PThread &, INT)
     threadStarted.Signal();
 }
 
+#endif // P_VFW_CAPTURE
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoOutputDevice_Window
@@ -1156,8 +1075,8 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
     PSyncPoint m_started;
     BITMAPINFO m_bitmap;
     bool       m_flipped;
-    int        m_lastX;
-    int        m_lastY;
+    POINT      m_lastPosition;
+    SIZE       m_fixedSize;
 };
 
 
@@ -1221,8 +1140,10 @@ PVideoOutputDevice_Window::PVideoOutputDevice_Window()
   m_hWnd = NULL;
   m_thread = NULL;
   m_flipped = PFalse;
-  m_lastX = 0;
-  m_lastY = 0;
+  m_lastPosition.x = 0;
+  m_lastPosition.y = 0;
+  m_fixedSize.cx = 0;
+  m_fixedSize.cy = 0;
 
   m_bitmap.bmiHeader.biSize = sizeof(m_bitmap.bmiHeader);
   m_bitmap.bmiHeader.biWidth = frameWidth;
@@ -1373,10 +1294,10 @@ void PVideoOutputDevice_Window::SetWindowSize()
   RECT rect;
   rect.top = 0;
   rect.left = 0;
-  rect.bottom = frameHeight;
-  rect.right = frameWidth;
+  rect.bottom = m_fixedSize.cy > 0 ? m_fixedSize.cy : frameHeight;
+  rect.right = m_fixedSize.cx > 0 ? m_fixedSize.cx : frameWidth;
   ::AdjustWindowRectEx(&rect, GetWindowLong(m_hWnd, GWL_STYLE), false, GetWindowLong(m_hWnd, GWL_EXSTYLE));
-  ::SetWindowPos(m_hWnd, NULL, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_NOMOVE|SWP_NOZORDER);
+  ::SetWindowPos(m_hWnd, HWND_TOP, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_NOMOVE);
 }
 
 
@@ -1397,8 +1318,8 @@ PBoolean PVideoOutputDevice_Window::FrameComplete()
 
 PBoolean PVideoOutputDevice_Window::GetPosition(int & x, int & y) const
 {
-  x = m_lastX;
-  y = m_lastY;
+  x = m_lastPosition.x;
+  y = m_lastPosition.y;
   return PTrue;
 }
 
@@ -1440,6 +1361,17 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
 }
 
 
+static int GetTokenValue(const PString & deviceName, const char * token, int defaultValue)
+{
+  PINDEX pos = deviceName.Find(token);
+  if (pos == P_MAX_INDEX)
+    return defaultValue;
+
+  pos += strlen(token);
+  return deviceName.Mid(pos).AsInteger();
+}
+
+
 void PVideoOutputDevice_Window::HandleDisplay(PThread &, INT)
 {
 #ifndef _WIN32_WCE
@@ -1475,17 +1407,29 @@ void PVideoOutputDevice_Window::HandleDisplay(PThread &, INT)
       title = PString(PString::Literal, quotedTitle);
     }
 
-    pos = deviceName.Find("X=");
-    m_lastX = pos != P_MAX_INDEX ? atoi(&deviceName[pos+2]) : CW_USEDEFAULT;
+    m_lastPosition.x = GetTokenValue(deviceName, "X=", CW_USEDEFAULT);
+    m_lastPosition.y = GetTokenValue(deviceName, "Y=", CW_USEDEFAULT);
+    m_fixedSize.cx   = GetTokenValue(deviceName, "WIDTH=", 0);
+    m_fixedSize.cy   = GetTokenValue(deviceName, "HEIGHT=", 0);
 
-    pos = deviceName.Find("Y=");
-    m_lastY = pos != P_MAX_INDEX ? atoi(&deviceName[pos+2]) : CW_USEDEFAULT;
+    if (m_lastPosition.x == CW_USEDEFAULT && m_lastPosition.y == CW_USEDEFAULT) {
+      if (hParent != NULL) {
+        RECT rect;
+        GetWindowRect(hParent, &rect);
+        m_lastPosition.x = (rect.right + rect.left - frameWidth)/2;
+        m_lastPosition.y = (rect.bottom + rect.top - frameHeight)/2;
+      }
+      else {
+        m_lastPosition.x = (GetSystemMetrics(SM_CXSCREEN) - frameWidth)/2;
+        m_lastPosition.y = (GetSystemMetrics(SM_CYSCREEN) - frameHeight)/2;
+      }
+    }
 
     PVarString windowTitle = title;
     m_hWnd = CreateWindow(wndClassName,
                           windowTitle, 
                           dwStyle,
-                          m_lastX, m_lastY, frameWidth, frameHeight,
+                          m_lastPosition.x, m_lastPosition.y, frameWidth, frameHeight,
                           hParent, NULL, GetModuleHandle(NULL), this);
     SetWindowSize();
   }
@@ -1518,8 +1462,8 @@ LRESULT PVideoOutputDevice_Window::WndProc(UINT uMsg, WPARAM wParam, LPARAM lPar
       if (m_hWnd != NULL) {
         RECT rect;
         GetWindowRect(m_hWnd, &rect);
-        m_lastX = rect.left;
-        m_lastY = rect.top;
+        m_lastPosition.x = rect.left;
+        m_lastPosition.y = rect.top;
       }
       break;
 
