@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21137 $
- * $Author: ms30002000 $
- * $Date: 2008-09-22 16:57:19 +0000 (Mon, 22 Sep 2008) $
+ * $Revision: 22295 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-26 00:31:31 +0000 (Thu, 26 Mar 2009) $
  */
 
 #pragma implementation "sockets.h"
@@ -67,6 +67,10 @@
 #include <netinet/if_ether.h>
 #endif
 
+#if defined(P_NETBSD)
+#include <ifaddrs.h>
+#endif
+
 #define ROUNDUP(a) \
         ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
 
@@ -471,6 +475,12 @@ PBoolean PIPSocket::IsLocalHost(const PString & hostname)
   PBYTEArray buffer;
   struct ifconf ifConf;
 
+#if defined(P_NETBSD)
+  struct ifaddrs *ifap, *ifa;
+
+  PAssert(getifaddrs(&ifap) == 0, "getifaddrs failed");
+  for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
+#else
 #ifdef SIOCGIFNUM
   int ifNum;
   PAssert(::ioctl(sock.GetHandle(), SIOCGIFNUM, &ifNum) >= 0, "could not do ioctl for ifNum");
@@ -486,8 +496,14 @@ PBoolean PIPSocket::IsLocalHost(const PString & hostname)
     ifreq * ifName = ifConf.ifc_req;
 
     while (ifName < ifEndList) {
+#endif
       struct ifreq ifReq;
+#if !defined(P_NETBSD)
       memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+      memset(&ifReq, 0, sizeof(ifReq));
+      strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
       
       if (ioctl(sock.GetHandle(), SIOCGIFFLAGS, &ifReq) >= 0) {
         int flags = ifReq.ifr_flags;
@@ -499,14 +515,16 @@ PBoolean PIPSocket::IsLocalHost(const PString & hostname)
         }
       }
       
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
       // move the ifName pointer along to the next ifreq entry
       ifName = (struct ifreq *)((char *)ifName + _SIZEOF_ADDR_IFREQ(*ifName));
-#else
+#elif !defined(P_NETBSD)
       ifName++;
 #endif
     }
+#if !defined(P_NETBSD)
   }
+#endif
   
   return PFalse;
 }
@@ -983,7 +1001,7 @@ PBoolean PEthSocket::Write(const void * buf, PINDEX len)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-PBoolean PIPSocket::GetGatewayAddress(Address & addr)
+PBoolean PIPSocket::GetGatewayAddress(Address & addr, int version)
 {
   RouteTable table;
   if (GetRouteTable(table)) {
@@ -999,7 +1017,7 @@ PBoolean PIPSocket::GetGatewayAddress(Address & addr)
 
 
 
-PString PIPSocket::GetGatewayInterface()
+PString PIPSocket::GetGatewayInterface(int version)
 {
   RouteTable table;
   if (GetRouteTable(table)) {
@@ -1017,34 +1035,38 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 {
   PTextFile procfile;
   if (!procfile.Open("/proc/net/route", PFile::ReadOnly))
-    return PFalse;
+    return false;
 
-  for (;;) {
+  PString strLine;
+  bool ok = procfile.ReadLine(strLine);
+  if (ok)
+    ok = procfile.ReadLine(strLine);
+
+  int nTime = 0;
+  while (ok) {
     // Ignore heading line or remainder of route line
-    procfile.ignore(1000, '\n');
-    if (procfile.eof())
-      return PTrue;
+    strLine.RightTrim();
+    PStringArray strList = strLine.Tokenise("\t");
+    if (strList.GetSize() < 8)
+      return nTime > 0 ? true : false;
 
-    char iface[20];
-    unsigned long net_addr, dest_addr, net_mask;
-    int flags, refcnt, use, metric;
-    procfile >> iface;
-    procfile.setf(hex, dec);
-    procfile >> net_addr >> dest_addr >> flags;
-    procfile.setf(dec, hex);
-    procfile >> refcnt >> use >> metric;
-    procfile.setf(hex, dec);
-    procfile >> net_mask;
-    if (procfile.bad())
-      return PFalse;
+    PString iface = strList[0];
+    unsigned net_addr = strList[1].AsUnsigned(16);
+    unsigned dest_addr = strList[2].AsUnsigned(16);
+    int metric= strList[6].AsInteger(10);
+    unsigned net_mask = strList[7].AsUnsigned(16);
 
+    nTime ++;
     RouteEntry * entry = new RouteEntry(net_addr);
     entry->net_mask = net_mask;
     entry->destination = dest_addr;
     entry->interfaceName = iface;
     entry->metric = metric;
     table.Append(entry);
+    ok = procfile.ReadLine(strLine);
   }
+
+  return nTime > 0;
 }
 
 #elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_QNX) 
@@ -1517,6 +1539,17 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 #endif
 
 
+bool PIPSocket::WaitForRouteTableChange(const PTimeInterval & timeout, PSyncPoint * cancellation)
+{
+  if (cancellation != NULL)
+    return cancellation->Wait(timeout);
+
+  // Need an implementation here!!!!!!!
+  PThread::Sleep(timeout);
+  return false;
+}
+
+
 // fe800000000000000202e3fffe1ee330 02 40 20 80     eth0
 // 00000000000000000000000000000001 01 80 10 80       lo
 
@@ -1559,7 +1592,13 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
   PBYTEArray buffer;
   struct ifconf ifConf;
   
+#if defined(P_NETBSD)
+  struct ifaddrs *ifap, *ifa;
 
+  PAssert(getifaddrs(&ifap) == 0, "getifaddrs failed");
+
+  for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
+#else
   // HERE
 #if defined(SIOCGIFNUM)
   int ifNum;
@@ -1575,9 +1614,14 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
     void * ifEndList = (char *)ifConf.ifc_req + ifConf.ifc_len;
     ifreq * ifName = ifConf.ifc_req;
     while (ifName < ifEndList) {
-
+#endif
       struct ifreq ifReq;
+#if !defined(P_NETBSD)
       memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+      memset(&ifReq, 0, sizeof(ifReq));
+      strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
 
       if (ioctl(sock.GetHandle(), SIOCGIFFLAGS, &ifReq) >= 0) {
         int flags = ifReq.ifr_flags;
@@ -1593,13 +1637,25 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
           }
 #endif
 
+#if !defined(P_NETBSD)
           memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+          memset(&ifReq, 0, sizeof(ifReq));
+          strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
+
           if (ioctl(sock.GetHandle(), SIOCGIFADDR, &ifReq) >= 0) {
 
             sockaddr_in * sin = (sockaddr_in *)&ifReq.ifr_addr;
             PIPSocket::Address addr = sin->sin_addr;
 
+#if !defined(P_NETBSD)
             memcpy(&ifReq, ifName, sizeof(ifreq));
+#else
+            memset(&ifReq, 0, sizeof(ifReq));
+            strncpy(ifReq.ifr_name, ifa->ifa_name, sizeof(ifReq.ifr_name) - 1);
+#endif
+
             if (ioctl(sock.GetHandle(), SIOCGIFNETMASK, &ifReq) >= 0) {
               PIPSocket::Address mask = 
 #ifndef P_BEOS
@@ -1637,15 +1693,17 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
         }
       }
 
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
       // move the ifName pointer along to the next ifreq entry
       ifName = (struct ifreq *)((char *)ifName + _SIZEOF_ADDR_IFREQ(*ifName));
-#else
+#elif !defined(P_NETBSD)
       ifName++;
 #endif
 
     }
+#if !defined(P_NETBSD)
   }
+#endif
   return PTrue;
 }
 
