@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 22295 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-03-26 00:31:31 +0000 (Thu, 26 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -165,7 +165,7 @@ void PSystemLog::Output(Level level, const char * cmsg)
 #elif defined(BE_THREADS)
       thread_id tid = ::find_thread(NULL);
 #else
-      unsigned tid = (unsigned) pthread_self();
+      unsigned tid = (uintptr_t) pthread_self();
 #endif
       *out << "ThreadID=0x"
            << setfill('0') << ::hex
@@ -339,7 +339,8 @@ int PServiceProcess::InitialiseService()
              "l-log-file:"
              "u-uid:"
              "g-gid:"
-             "C-core-size:");
+             "C-core-size:"
+           , false);
 
   // if only displaying version information, do it and finish
   if (args.HasOption('v')) {
@@ -719,7 +720,7 @@ void PServiceProcess::PXOnAsyncSignal(int sig)
 #elif defined(BE_THREADS)
   thread_id tid = ::find_thread(NULL);
 #else
-  unsigned tid = (unsigned) pthread_self();
+  unsigned tid = (uintptr_t) pthread_self();
 #endif
   PThread * thread_ptr = activeThreads.GetAt(tid);
 
